#!/bin/bash
###################################################################################
#
#  fwadsl 2005 v3
#
#  Load balance script
#
###################################################################################

ip=`which ip`
ipcalc=`which ipcalc`
iptables=`which iptables`
rttables="/etc/iproute2/rt_tables"
rtprio=80

delete()
{
   ### Delete default gw
   [ "$1" == "default" ] && \
      $ip route delete default 2>/dev/null
}

define()
{
   local ALIAS=$1
   local ID=$2
   
   ### Update routing table file (/etc/iproute2/rt_tables)
   [ "`cat /etc/iproute2/rt_tables | grep $ALIAS | awk '{print $2}'`"id == id ] && \
       echo $ID $ALIAS >> $rttables
      
   ### Flush routing table
   $ip route flush table $ALIAS 2> /dev/null
}

config()
{
   local ctable=$1
   local ceth=$2
   local cipaddr=$3
   local cgateway=$6
   local cnet=$($ipcalc -n $cipaddr"$4" | cut -d"=" -f2-)
   cnet=$cnet"$4"
 
   ### Include a new interface route
   newrt=$($ip route ls dev $ceth | grep $cipaddr | wc -l)
    
   ### Adding a ethernet ip address 
   $ip addr del $cipaddr"$4" brd + dev $ceth 2>/dev/null
   $ip addr add $cipaddr"$4" brd + dev $ceth 2>/dev/null
     
   ### Make routing tables and default gateway 
   [ "$newrt" -eq 0 ] && \
      $ip route add $cnet dev $ceth src $cipaddr proto static 2>/dev/null || \
      $ip route ch $cnet dev $ceth src $cipaddr proto static 2>/dev/null
      
   $ip route add default via $cgateway dev $ceth src $cipaddr proto static table $ctable 2>/dev/null
   $ip route append prohibit default table $ctable metric 1 proto static 2>/dev/null

   ### Make routing rules
   $ip rule del prio $rtprio from $cnet table $ctable 2>/dev/null
   $ip rule add prio $rtprio from $cnet table $ctable 2>/dev/null

   ((rtprio++))

   $iptables -t nat -D POSTROUTING -o $ceth -s $cipaddr -j SNAT --to $cipaddr 2>/dev/null 
   $iptables -t nat -I POSTROUTING -o $ceth -s $cipaddr -j SNAT --to $cipaddr 2>/dev/null
}

balance()
{
   ### Prio for table "main"
   $ip rule del prio 10 table main 2> /dev/null
   $ip rule add prio 10 table main 2> /dev/null

   echo "$ip route add default $equal proto static scope global table $1 \\" > /tmp/mpath.$$
   
   ((rtprio+=10))
   $ip rule del prio $rtprio table $1 2>/dev/null
   $ip rule add prio $rtprio table $1 2>/dev/null
   
   shift
   cont=1
   for i in $(echo $@);
   do
      aux_tb=$(echo $i | cut -d":" -f1)
      aux_we=$(echo $i | cut -d":" -f2)
      aux_gw=$($ip route ls table $aux_tb | grep via | cut -d" " -f 3)
      [ "$cont" -lt "$#" ] && nlin="\\" || nlin=" "
      echo "   nexthop via $aux_gw weight $aux_we $nlin" >> /tmp/mpath.$$
      ((cont++))
   done
   sh /tmp/mpath.$$
   rm -rf /tmp/mpath.$$
   echo "Firewall... Load balance started "
}

timeout()
{
   local ccmd=$1
   [ "$ccmd" == "gateway" ] && echo $2 > /proc/sys/net/ipv4/route/gc_timeout
   [ "$ccmd" == "rtcache" ] && \
   { 
      echo $2 > /proc/sys/net/ipv4/route/gc_interval
      echo $2 > /proc/sys/net/ipv4/route/gc_min_interval
   }
}

route()
{
   local ccmd=$1
   ((rtprio++))
   [ "$ccmd" == "to" ] && \
   {
      $ip rule del prio $rtprio to $2 table $3 2>/dev/null
      $ip rule add prio $rtprio to $2 table $3 2>/dev/null
   } 
      
   [ "$ccmd" == "from" ] && \
   {
      $ip rule del prio $rtprio from $2 table $3 2>/dev/null
      $ip rule add prio $rtprio from $2 table $3 2>/dev/null
   }
}

while read linha;
do
   cmd="$(echo $linha | awk '{print $1}')"
   cmdparam=$(echo $linha | cut -d" " -f2- )
   
   [ "$cmd" == "timeout" ] && timeout $cmdparam
   [ "$cmd" == "delete" ] && delete $cmdparam
   [ "$cmd" == "define" ] && define $cmdparam
   [ "$cmd" == "config" ] && config $cmdparam
   [ "$cmd" == "route" ] && route $cmdparam  
   [ "$cmd" == "balance" ] && \
   {
      equal=""
      balance $cmdparam
   }
   
   [ "$cmd" == "eqbalance" ] && \
   {
      equal="equalize"
      balance $cmdparam
   }
   
done < $(dirname $0)/config.mpath

unset rtprio

